@@ -61,6 +61,8 @@ pyab validate --connector=source-hardcoded-records --config='{count: 400_000}'
 
 from __future__ import annotations
 
+import re
+import sys
 from pathlib import Path
 from typing import TYPE_CHECKING, Any
 
@@ -173,12 +175,43 @@ def _get_connector_name(connector: str) -> str:
     return connector
 
 
+def _parse_use_python(use_python_str: str | None) -> bool | Path | str | None:
+    r"""Parse the use_python CLI parameter.
+
+    Args:
+        use_python_str: The raw string value from CLI input.
+
+    Returns:
+        - None: No parameter provided
+        - True: Use current Python interpreter ("true")
+        - False: Use Docker instead ("false")
+        - Path: Use interpreter at this path (paths containing / or \ or starting with .)
+        - str: Use uv-managed Python version (semver patterns like "3.12", "3.11.5")
+               or existing interpreter name (non-semver strings like "python3.10")
+    """
+    if use_python_str is None:
+        return None
+    if use_python_str.lower() == "true":
+        return True
+    if use_python_str.lower() == "false":
+        return False
+    if "/" in use_python_str or "\\" in use_python_str or use_python_str.startswith("."):
+        return Path(use_python_str)
+
+    semver_pattern = r"^\d+\.\d+(?:\.\d+)?$"
+    if re.match(semver_pattern, use_python_str):
+        return use_python_str  # Return as string for uv-managed version
+
+    return Path(use_python_str)
+
+
 def _resolve_source_job(
     *,
     source: str | None = None,
     config: str | None = None,
     streams: str | None = None,
     pip_url: str | None = None,
+    use_python: str | None = None,
 ) -> Source:
     """Resolve the source job into a configured Source object.
 
@@ -190,12 +223,15 @@ def _resolve_source_job(
         streams: A comma-separated list of stream names to select for reading. If set to "*",
             all streams will be selected. If not provided, all streams will be selected.
         pip_url: Optional. A location from which to install the connector.
+        use_python: Optional. Python interpreter specification.
     """
     config_dict = _resolve_config(config) if config else None
     streams_list: str | list[str] = streams or "*"
     if isinstance(streams, str) and streams != "*":
         streams_list = [stream.strip() for stream in streams.split(",")]
 
+    use_python_parsed = _parse_use_python(use_python)
+
     source_obj: Source
     if source and _is_docker_image(source):
         source_obj = get_source(
@@ -204,6 +240,7 @@ def _resolve_source_job(
             config=config_dict,
             streams=streams_list,
             pip_url=pip_url,
+            use_python=use_python_parsed,
         )
         return source_obj
 
@@ -223,6 +260,7 @@ def _resolve_source_job(
             config=config_dict,
             streams=streams_list,
             pip_url=pip_url,
+            use_python=use_python_parsed,
         )
         return source_obj
 
@@ -239,14 +277,24 @@ def _resolve_source_job(
         config=config_dict,
         streams=streams_list,
         pip_url=pip_url,
+        use_python=use_python_parsed,
     )
 
 
+def _get_noop_destination_config() -> dict[str, Any]:
+    return {
+        "test_destination": {
+            "test_destination_type": "SILENT",
+        }
+    }
+
+
 def _resolve_destination_job(
     *,
     destination: str,
     config: str | None = None,
     pip_url: str | None = None,
+    use_python: str | None = None,
 ) -> Destination:
     """Resolve the destination job into a configured Destination object.
 
@@ -257,8 +305,24 @@ def _resolve_destination_job(
             and tag.
         config: The path to a configuration file for the named source or destination.
         pip_url: Optional. A location from which to install the connector.
+        use_python: Optional. Python interpreter specification.
     """
-    config_dict = _resolve_config(config) if config else None
+    config_dict = _resolve_config(config) if config else {}
+    use_python_parsed = _parse_use_python(use_python)
+
+    destination_name = _get_connector_name(destination)
+
+    if destination_name == "destination-dev-null" and not config:
+        config_dict = _get_noop_destination_config()
+
+    if _is_docker_image(destination):
+        return get_destination(
+            name=destination_name,
+            docker_image=destination,
+            config=config_dict,
+            pip_url=pip_url,
+            use_python=use_python_parsed,
+        )
 
     if destination and (destination.startswith(".") or "/" in destination):
         # Treat the destination as a path.
@@ -275,6 +339,7 @@ def _resolve_destination_job(
             local_executable=destination_executable,
             config=config_dict,
             pip_url=pip_url,
+            use_python=use_python_parsed,
         )
 
     # else: # Treat the destination as a name.
@@ -283,6 +348,7 @@ def _resolve_destination_job(
         name=destination,
         config=config_dict,
         pip_url=pip_url,
+        use_python=use_python_parsed,
     )
 
 
@@ -313,10 +379,20 @@ def _resolve_destination_job(
     required=False,
     help=CONFIG_HELP,
 )
+@click.option(
+    "--use-python",
+    type=str,
+    help=(
+        "Python interpreter specification. Use 'true' for current Python, "
+        "'false' for Docker, a path for specific interpreter, or a version "
+        "string for uv-managed Python (e.g., '3.11', 'python3.12')."
+    ),
+)
 def validate(
     connector: str | None = None,
     config: str | None = None,
     pip_url: str | None = None,
+    use_python: str | None = None,
 ) -> None:
     """CLI command to run a `benchmark` operation."""
     if not connector:
@@ -331,16 +407,18 @@ def validate(
             config=None,
             streams=None,
             pip_url=pip_url,
+            use_python=use_python,
         )
     else:  # destination
         connector_obj = _resolve_destination_job(
             destination=connector,
             config=None,
             pip_url=pip_url,
+            use_python=use_python,
         )
 
-    print("Getting `spec` output from connector...")
-    connector_obj.print_config_spec()
+    print("Getting `spec` output from connector...", file=sys.stderr)
+    connector_obj.print_config_spec(stderr=True)
 
     if config:
         print("Running connector check...")
@@ -392,12 +470,22 @@ def validate(
     type=str,
     help=CONFIG_HELP,
 )
+@click.option(
+    "--use-python",
+    type=str,
+    help=(
+        "Python interpreter specification. Use 'true' for current Python, "
+        "'false' for Docker, a path for specific interpreter, or a version "
+        "string for uv-managed Python (e.g., '3.11', 'python3.12')."
+    ),
+)
 def benchmark(
     source: str | None = None,
     streams: str = "*",
     num_records: int | str = "5e5",  # 500,000 records
     destination: str | None = None,
     config: str | None = None,
+    use_python: str | None = None,
 ) -> None:
     """CLI command to run a `benchmark` operation.
 
@@ -420,6 +508,7 @@ def benchmark(
             source=source,
             config=config,
             streams=streams,
+            use_python=use_python,
         )
         if source
         else get_benchmark_source(
@@ -430,12 +519,13 @@ def benchmark(
         _resolve_destination_job(
             destination=destination,
             config=config,
+            use_python=use_python,
         )
         if destination
         else get_noop_destination()
     )
 
-    click.echo("Running benchmarks...")
+    click.echo("Running benchmarks...", sys.stderr)
     destination_obj.write(
         source_data=source_obj,
         cache=False,
@@ -492,6 +582,15 @@ def benchmark(
     type=str,
     help="Optional pip URL for the destination (Python connectors only). " + PIP_URL_HELP,
 )
+@click.option(
+    "--use-python",
+    type=str,
+    help=(
+        "Python interpreter specification. Use 'true' for current Python, "
+        "'false' for Docker, a path for specific interpreter, or a version "
+        "string for uv-managed Python (e.g., '3.11', 'python3.12')."
+    ),
+)
 def sync(
     *,
     source: str,
@@ -501,6 +600,7 @@ def sync(
     destination_config: str | None = None,
     destination_pip_url: str | None = None,
     streams: str | None = None,
+    use_python: str | None = None,
 ) -> None:
     """CLI command to run a `sync` operation.
 
@@ -515,11 +615,13 @@ def sync(
         config=source_config,
         streams=streams,
         pip_url=source_pip_url,
+        use_python=use_python,
     )
     destination_obj = _resolve_destination_job(
         destination=destination,
         config=destination_config,
         pip_url=destination_pip_url,
+        use_python=use_python,
     )
 
     click.echo("Running sync...")
